
// Qt include.
#include <QtCore/QList>
#include <QtCore/QTimer>
#include <QtCore/QDebug>
#include <QtCore/QMutex>
#include <QtCore/QList>

#include "server_socket.hpp"
#include "client_socket.hpp"

//
// MutexLocker
//

class MutexLocker {
public:
	MutexLocker( QMutex * mutex )
		:	m_mutex( mutex )
	{
		qDebug() << "Mutex locked";

		m_mutex->lock();
	}

	~MutexLocker()
	{
		qDebug() << "Mutex unlocked";

		m_mutex->unlock();
	}

private:
	QMutex * m_mutex;
};


//
// ServerSocketPrivate
//

struct ServerSocket::ServerSocketPrivate {
	QMutex m_mutex;
	QList< ClientSocket* > m_clients;
};


//
// ServerSocket
//

ServerSocket::ServerSocket( QObject * parent )
	:	QTcpServer( parent )
	,	d( new ServerSocketPrivate )
{
}

ServerSocket::~ServerSocket()
{
}

void
ServerSocket::incomingConnection( int socketDescriptor )
{
	ClientSocket * socket = new ClientSocket( this );

	if( socket->setSocketDescriptor( socketDescriptor ) )
	{
		connect( socket, SIGNAL( disconnected() ),
			this, SLOT( slotDisconnected() ) );

		{
			MutexLocker lock( &d->m_mutex );

			d->m_clients.append( socket );
		}
	}
	else
		delete socket;
}

void
ServerSocket::slotDisconnected()
{
	qDebug() << "slotDisconnected()";

	MutexLocker lock( &d->m_mutex );

	sender()->deleteLater();
}

void
ServerSocket::doThis()
{
	qDebug() << "doThis()";

	MutexLocker lock( &d->m_mutex );

	foreach( ClientSocket * socket, d->m_clients )
		socket->sendData();
}
